import wx
from . import config
from TimesheetExcelEditor.Excel import Excel


class CreatePanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

        vbox = wx.BoxSizer(wx.VERTICAL)
        createBox = wx.BoxSizer(wx.HORIZONTAL)
        minLabel = wx.StaticText(self, label="Anno Min:")
        createBox.Add(minLabel, proportion=0, flag=wx.ALIGN_LEFT | wx.ALL ^ wx.BOTTOM, border=10)
        self.yearMin = wx.TextCtrl(self)
        self.yearMin.SetHint("AAAA")
        self.yearMin.Bind(wx.EVT_CHAR, self.__handle_keypress)
        createBox.Add(self.yearMin, proportion=1, flag=wx.ALL ^ wx.LEFT, border=10)
        addLabel = wx.StaticText(self, label="Aggiungi:")
        createBox.Add(addLabel, proportion=0, flag=wx.ALL, border=10)
        self.yearAdd = wx.ComboBox(self, choices=["1", "2", "3", "4", "5"],
                                   style=wx.CB_DROPDOWN | wx.CB_READONLY)
        self.yearAdd.SetSelection(0)
        createBox.Add(self.yearAdd, proportion=1, flag=wx.ALL ^ wx.LEFT, border=10)
        createButton = wx.Button(self, label='Crea')
        createButton.Bind(wx.EVT_BUTTON, self.__create_button_press)
        createBox.Add(createButton, proportion=0, flag=wx.ALL, border=10)
        vbox.Add(createBox, proportion=0, flag=wx.EXPAND | wx.ALL, border=5)

        self.SetSizer(vbox)

    def __handle_keypress(self, event):
        keycode = event.GetKeyCode()
        if keycode < 255:
            # valid ASCII
            if chr(keycode).isnumeric() or keycode == 8:
                # Valid numeric character
                event.Skip()

    def __create_button_press(self, event):
        if not self.yearMin.IsEmpty() and self.yearMin.GetValue().isnumeric():
            wb = Excel().createWorkbook(self.yearMin.GetValue(), int(self.yearAdd.GetValue()))
            saveFileDialog = wx.FileDialog(self, "Save", "", "*.xlsx", "Exel files (*.xlsx)|*.xlsx",
                                           wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
            if saveFileDialog.ShowModal() == wx.ID_CANCEL:
                return
            wb.save(saveFileDialog.GetPath())
            config.log.AppendText("File "+saveFileDialog.GetFilename()+" creato e salvato in "+saveFileDialog.GetPath()+"\n")
        else:
            missYearMin = wx.MessageDialog(None, "Inserire anno minimo", caption="Errore",
                                           style=wx.OK, pos=wx.DefaultPosition)
            if missYearMin.ShowModal() == wx.OK:
                return



import wx
from . import config


class FormPanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        vbox = wx.BoxSizer(wx.VERTICAL)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        dl = wx.StaticText(self, label="Data")
        hbox.Add(dl, proportion=0, flag=wx.ALIGN_LEFT | wx.ALL, border=10)
        self.data = wx.TextCtrl(self)
        self.data.SetHint("gg/mm/aaaa")
        hbox.Add(self.data, proportion=1, flag=wx.ALL, border=10)
        nol = wx.StaticText(self, label="Numero ore")
        hbox.Add(nol, proportion=0, flag=wx.ALL, border=10)
        self.numOre = wx.TextCtrl(self)
        self.numOre.Bind(wx.EVT_CHAR, self.__handle_keypress)
        hbox.Add(self.numOre, proportion=1, flag=wx.ALL, border=10)
        config.formModifyButton = wx.Button(self, label='Modifica')
        config.formModifyButton.Bind(wx.EVT_BUTTON, self.__form_modify_button_press)
        config.formModifyButton.Disable()
        hbox.Add(config.formModifyButton, proportion=0, flag=wx.ALL, border=10)
        self.saveButton = wx.Button(self, label="Salva")
        self.saveButton.Bind(wx.EVT_BUTTON, self.__save_button_press)
        self.saveButton.Disable()
        hbox.Add(self.saveButton, proportion=0, flag=wx.ALL, border=10)

        vbox.Add(hbox, proportion=1, flag=wx.EXPAND | wx.ALL, border=10)
        self.SetSizer(vbox)

    def __handle_keypress(self, event):
        keycode = event.GetKeyCode()
        if keycode < 255:
            # valid ASCII
            if chr(keycode).isnumeric() or keycode == 8:
                # Valid numeric character
                event.Skip()

    def __form_modify_button_press(self, event):
        if not self.data.IsEmpty() and config.checkDate(self.data.GetValue()):
            if not self.numOre.IsEmpty():
                multiplier = config.multiplier.GetStringSelection()
                activity = config.acSelect.GetStringSelection()
                config.file.modify(activity, self.data.GetValue(), int(self.numOre.GetValue()), float(multiplier))
                modified = wx.MessageDialog(None, "Modificato", caption="Info",
                                            style=wx.OK, pos=wx.DefaultPosition)
                if modified.ShowModal() == wx.OK:
                    return
                self.saveButton.Enable()
                if config.file.checkMissing():
                    message = config.file.getMissing()
                    config.log.AppendText(message+"\n")
            else:
                missHours = wx.MessageDialog(None, "Inserire un numero di ore", caption="Errore",
                                             style=wx.OK, pos=wx.DefaultPosition)
                if missHours.ShowModal() == wx.OK:
                    return
        else:
            missDate = wx.MessageDialog(None, "Data mancante o non valida", caption="Errore",
                                        style=wx.OK, pos=wx.DefaultPosition)
            if missDate.ShowModal() == wx.OK:
                return

    def __save_button_press(self, event):
        file = config.file
        saveFileDialog = wx.FileDialog(self, "Save", "", config.file.getFileName(), "Exel files (*.xlsx)|*.xlsx",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if saveFileDialog.ShowModal() == wx.ID_CANCEL:
            return
        file.save(saveFileDialog.GetPath())



import re
from datetime import datetime


file = None
log = None
acSelect = None
multiplier = None
calModifyButton = None
listModifyButton = None
formModifyButton = None
rangeModifyButton = None


def checkDate(date):
    if not re.match("\d{1,2}/\d{2}/\d{4}", date):
        return False
    day, month, year = date.split('/')
    isValidDate = True
    try:
        datetime(int(year), int(month), int(day))
    except ValueError:
        isValidDate = False
    return isValidDate



import wx
from . import config
from datetime import datetime as dt, timedelta


class RangePanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

        vbox = wx.BoxSizer(wx.VERTICAL)
        rangeBox = wx.BoxSizer(wx.HORIZONTAL)
        minLabel = wx.StaticText(self, label="Data Min:")
        rangeBox.Add(minLabel, proportion=0, flag=wx.ALIGN_LEFT | wx.ALL ^ wx.BOTTOM, border=10)
        self.dataMin = wx.TextCtrl(self)
        self.dataMin.SetHint("gg/mm/aaaa")
        rangeBox.Add(self.dataMin, proportion=1, flag=wx.EXPAND | wx.ALL ^ wx.LEFT, border=10)
        maxLabel = wx.StaticText(self, label="Data Max:")
        rangeBox.Add(maxLabel, proportion=0, flag=wx.ALL, border=10)
        self.dataMax = wx.TextCtrl(self)
        self.dataMax.SetHint("gg/mm/aaaa")
        rangeBox.Add(self.dataMax, proportion=1, flag=wx.EXPAND | wx.ALL ^ wx.LEFT, border=10)
        config.rangeModifyButton = wx.Button(self, label='Modifica')
        config.rangeModifyButton.Bind(wx.EVT_BUTTON, self.__range_modify_button_press)
        config.rangeModifyButton.Disable()
        rangeBox.Add(config.rangeModifyButton, proportion=0, flag=wx.ALL, border=10)
        self.saveButton = wx.Button(self, label="Salva")
        self.saveButton.Bind(wx.EVT_BUTTON, self.__save_button_press)
        self.saveButton.Disable()
        rangeBox.Add(self.saveButton, proportion=0, flag=wx.ALL ^ wx.LEFT, border=10)
        vbox.Add(rangeBox, proportion=0, flag=wx.EXPAND | wx.ALL, border=5)

        mondayBox = wx.BoxSizer(wx.HORIZONTAL)
        mLabel = wx.StaticText(self, label="Lunedì")
        mondayBox.Add(mLabel, proportion=0, flag=wx.ALIGN_LEFT | wx.ALL ^ wx.TOP, border=10)
        self.mondayHours = wx.TextCtrl(self)
        self.mondayHours.Bind(wx.EVT_CHAR, self.__handle_keypress)
        self.mondayHours.SetHint("Num ore")
        mondayBox.Add(self.mondayHours, proportion=0, flag=wx.ALL ^ wx.TOP ^ wx.BOTTOM, border=10)
        vbox.Add(mondayBox, proportion=0, flag=wx.ALL, border=5)

        tuesdayBox = wx.BoxSizer(wx.HORIZONTAL)
        tLabel = wx.StaticText(self, label="Martedì")
        tuesdayBox.Add(tLabel, proportion=0, flag=wx.ALL ^ wx.TOP, border=10)
        self.tuesdayHours = wx.TextCtrl(self)
        self.tuesdayHours.SetHint("Num ore")
        self.tuesdayHours.Bind(wx.EVT_CHAR, self.__handle_keypress)
        tuesdayBox.Add(self.tuesdayHours, proportion=0, flag=wx.ALL ^ wx.TOP, border=5)
        vbox.Add(tuesdayBox, proportion=0, flag=wx.ALL, border=5)

        wednesdayBox = wx.BoxSizer(wx.HORIZONTAL)
        mLabel = wx.StaticText(self, label="Mercoledì")
        wednesdayBox.Add(mLabel, proportion=0, flag=wx.ALL ^ wx.TOP, border=5)
        self.wednesdayHours = wx.TextCtrl(self)
        self.wednesdayHours.SetHint("Num ore")
        self.wednesdayHours.Bind(wx.EVT_CHAR, self.__handle_keypress)
        wednesdayBox.Add(self.wednesdayHours, proportion=0, flag=wx.ALL ^ wx.TOP, border=5)
        vbox.Add(wednesdayBox, proportion=0, flag=wx.ALL, border=5)

        thursdayBox = wx.BoxSizer(wx.HORIZONTAL)
        mLabel = wx.StaticText(self, label="Giovedì")
        thursdayBox.Add(mLabel, proportion=0, flag=wx.ALL ^ wx.TOP, border=10)
        self.thursdayHours = wx.TextCtrl(self)
        self.thursdayHours.SetHint("Num ore")
        self.thursdayHours.Bind(wx.EVT_CHAR, self.__handle_keypress)
        thursdayBox.Add(self.thursdayHours, proportion=0, flag=wx.ALL ^ wx.TOP, border=5)
        vbox.Add(thursdayBox, proportion=0, flag=wx.ALL, border=5)

        fridayBox = wx.BoxSizer(wx.HORIZONTAL)
        fLabel = wx.StaticText(self, label="Venerdi")
        fridayBox.Add(fLabel, proportion=0, flag=wx.ALL ^ wx.TOP, border=10)
        self.fridayHours = wx.TextCtrl(self)
        self.fridayHours.SetHint("Num ore")
        self.fridayHours.Bind(wx.EVT_CHAR, self.__handle_keypress)
        fridayBox.Add(self.fridayHours, proportion=0, flag=wx.ALL ^ wx.TOP, border=5)
        vbox.Add(fridayBox, proportion=0, flag=wx.ALL, border=5)

        saturdayBox = wx.BoxSizer(wx.HORIZONTAL)
        sLabel = wx.StaticText(self, label="Sabato")
        fridayBox.Add(sLabel, proportion=0, flag=wx.ALIGN_TOP | wx.ALL ^ wx.TOP, border=10)
        self.saturdayHours = wx.TextCtrl(self)
        self.saturdayHours.SetHint("Num ore")
        self.saturdayHours.Bind(wx.EVT_CHAR, self.__handle_keypress)
        fridayBox.Add(self.saturdayHours, proportion=0, flag=wx.ALL ^ wx.TOP, border=5)
        vbox.Add(saturdayBox, proportion=0, flag=wx.ALL, border=5)

        self.SetSizer(vbox)

    def __handle_keypress(self, event):
        keycode = event.GetKeyCode()
        if keycode < 255:
            # valid ASCII
            if chr(keycode).isnumeric() or keycode == 8:
                # Valid numeric character
                event.Skip()

    def __range_modify_button_press(self, event):
        activity = config.acSelect.GetStringSelection()
        multiplier = config.multiplier.GetStringSelection()
        if not self.dataMin.IsEmpty() and config.checkDate(self.dataMin.GetValue()):
            if not self.dataMax.IsEmpty() and config.checkDate(self.dataMin.GetValue()):
                monday = tuesday = wednesday = thursday = friday = saturday = None
                rangeStart = dt.strptime(self.dataMin.GetValue(), "%d/%m/%Y")
                rangeEnd = dt.strptime(self.dataMax.GetValue(), "%d/%m/%Y")
                if not self.mondayHours.IsEmpty():
                    monday = self.mondayHours.GetValue()
                if not self.tuesdayHours.IsEmpty():
                    tuesday = self.tuesdayHours.GetValue()
                if not self.wednesdayHours.IsEmpty():
                    wednesday = self.wednesdayHours.GetValue()
                if not self.thursdayHours.IsEmpty():
                    thursday = self.thursdayHours.GetValue()
                if not self.fridayHours.IsEmpty():
                    friday = self.fridayHours.GetValue()
                if not self.saturdayHours.IsEmpty():
                    saturday = self.saturdayHours.GetValue()
                while rangeEnd >= rangeStart:
                    if monday is not None:
                        if rangeStart.strftime("%A") == "Monday":
                            d = rangeStart.strftime("%d/%m/%Y")
                            config.file.modify(activity, d, int(monday), float(multiplier))
                    if tuesday is not None:
                        if rangeStart.strftime("%A") == "Tuesday":
                            d = rangeStart.strftime("%d/%m/%Y")
                            config.file.modify(activity, d, int(tuesday), float(multiplier))
                    if wednesday is not None:
                        if rangeStart.strftime("%A") == "Wednesday":
                            d = rangeStart.strftime("%d/%m/%Y")
                            config.file.modify(activity, d, int(wednesday), float(multiplier))
                    if thursday is not None:
                        if rangeStart.strftime("%A") == "Thursday":
                            d = rangeStart.strftime("%d/%m/%Y")
                            config.file.modify(activity, d, int(thursday), float(multiplier))
                    if friday is not None:
                        if rangeStart.strftime("%A") == "Friday":
                            d = rangeStart.strftime("%d/%m/%Y")
                            config.file.modify(activity, d, int(friday), float(multiplier))
                    if saturday is not None:
                        if rangeStart.strftime("%A") == "Saturday":
                            d = rangeStart.strftime("%d/%m/%Y")
                            config.file.modify(activity, d, int(saturday), float(multiplier))
                    rangeStart = rangeStart + timedelta(days=1)
                modified = wx.MessageDialog(None, "File modificato", caption="Info",
                                            style=wx.OK, pos=wx.DefaultPosition)
                if modified.ShowModal() == wx.OK:
                    return
                if config.file.checkMissing():
                    message = config.file.getMissing()
                    config.log.AppendText(message + "\n")
                self.saveButton.Enable()
            else:
                missDateMax = wx.MessageDialog(None, "Inserire data massima", caption="Errore",
                                               style=wx.OK, pos=wx.DefaultPosition)
                if missDateMax.ShowModal() == wx.OK:
                    return
        else:
            missDateMin = wx.MessageDialog(None, "Inserire data minima", caption="Errore",
                                         style=wx.OK, pos=wx.DefaultPosition)
            if missDateMin.ShowModal() == wx.OK:
                return

    def __save_button_press(self, event):
        file = config.file
        saveFileDialog = wx.FileDialog(self, "Save", "", config.file.getFileName(), "Exel files (*.xlsx)|*.xlsx",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if saveFileDialog.ShowModal() == wx.ID_CANCEL:
            return
        file.save(saveFileDialog.GetPath())



import wx
import re
from . import config


class ListPanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        vbox = wx.BoxSizer(wx.VERTICAL)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        listlabel = wx.StaticText(self, label="Elenco")
        hbox.Add(listlabel, proportion=0, flag=wx.ALIGN_LEFT | wx.ALL, border=10)
        self.elenco = wx.TextCtrl(self, style=wx.TE_MULTILINE | wx.SUNKEN_BORDER)
        hbox.Add(self.elenco, proportion=1, flag=wx.EXPAND | wx.ALL, border=10)
        config.listModifyButton = wx.Button(self, label='Modifica')
        config.listModifyButton.Bind(wx.EVT_BUTTON, self.__list_modify_button_press)
        config.listModifyButton.Disable()
        hbox.Add(config.listModifyButton, proportion=0, flag=wx.ALL, border=10)
        self.saveButton = wx.Button(self, label="Salva")
        self.saveButton.Bind(wx.EVT_BUTTON, self.__save_button_press)
        self.saveButton.Disable()
        hbox.Add(self.saveButton, proportion=0, flag=wx.ALL, border=10)
        vbox.Add(hbox, proportion=1, flag=wx.EXPAND | wx.ALL, border=10)

        self.SetSizer(vbox)

    def __checkList(self):
        num = self.elenco.GetNumberOfLines()
        for i in range(num):
            line = self.elenco.GetLineText(i)
            if not re.match("\d{1,2}/\d{2}/\d{4} \d+", line):
                return False
            date, hours = line.split(" ")
            if not config.checkDate(date):
                return False
        return True

    def __list_modify_button_press(self, event):
        activity = config.acSelect.GetStringSelection()
        multiplier = config.multiplier.GetStringSelection()
        if not self.elenco.IsEmpty() and self.__checkList():
            num = self.elenco.GetNumberOfLines()
            for i in range(num):
                line = self.elenco.GetLineText(i)
                date, hours = line.split(" ")
                config.file.modify(activity, date, int(hours), float(multiplier))
            modified = wx.MessageDialog(None, "File modificato", caption="Info",
                                        style=wx.OK, pos=wx.DefaultPosition)
            if modified.ShowModal() == wx.OK:
                return
            if config.file.checkMissing():
                message = config.file.getMissing()
                config.log.AppendText(message+"\n")
            self.saveButton.Enable()
        else:
            missList = wx.MessageDialog(None, "Inserire elenco formato:data nOre", caption="Errore",
                                        style=wx.OK, pos=wx.DefaultPosition)
            if missList.ShowModal() == wx.OK:
                return

    def __save_button_press(self, event):
        file = config.file
        saveFileDialog = wx.FileDialog(self, "Save", "", config.file.getFileName(), "Exel files (*.xlsx)|*.xlsx",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if saveFileDialog.ShowModal() == wx.ID_CANCEL:
            return
        file.save(saveFileDialog.GetPath())



from setuptools import setup
import pathlib

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='TimesheetExcelEditor',
    version='1.1.1',
    description='An application for modify excel timesheet',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/alaino95/TimesheetExcelEditor',
    license='MIT',
    author='Alessandro Laino',
    author_email='alessandrolaino1@gmail.com',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
    packages=['TimesheetExcelEditor'],
    install_requires=["wxpython",
                      "openpyxl",
                      "icalendar",
                      "validators",
                      "python-dateutil"
                      ],
)



import wx
import re
import validators
from icalendar import Calendar
from datetime import datetime
from dateutil.relativedelta import relativedelta
import dateutil.rrule as rrule
from urllib.request import Request, urlopen
from . import config


class CalendarPanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

        svbox = wx.BoxSizer(wx.HORIZONTAL)

        leftpanel = wx.Panel(self)
        vbox = wx.BoxSizer(wx.VERTICAL)

        linkhbox = wx.BoxSizer(wx.HORIZONTAL)
        linklabel = wx.StaticText(leftpanel, label="Link")
        linkhbox.Add(linklabel, proportion=0, flag=wx.ALIGN_LEFT | wx.RIGHT, border=10)
        self.link = wx.TextCtrl(leftpanel)
        linkhbox.Add(self.link, proportion=1, flag=wx.LEFT | wx.RIGHT, border=10)
        vbox.Add(linkhbox, proportion=0, flag=wx.EXPAND | wx.ALL, border=10)

        filterBox = wx.BoxSizer(wx.HORIZONTAL)
        filterLabel = wx.StaticText(leftpanel, label="Filtri")
        filterBox.Add(filterLabel, proportion=0, flag=wx.ALIGN_LEFT | wx.RIGHT, border=15)
        self.nameFilter = wx.TextCtrl(leftpanel)
        self.nameFilter.SetHint("Nome...")
        self.dataMin = wx.TextCtrl(leftpanel)
        self.dataMin.SetHint("Data Min...")
        self.dataMax = wx.TextCtrl(leftpanel)
        self.dataMax.SetHint("Data Max...")
        self.numOre = wx.TextCtrl(leftpanel)
        self.numOre.SetHint("Num ore...")
        self.numOre.Bind(wx.EVT_CHAR, self.__handle_keypress)
        filterBox.Add(self.nameFilter, proportion=1, flag=wx.LEFT | wx.RIGHT, border=4)
        filterBox.Add(self.dataMin, proportion=1, flag=wx.LEFT | wx.RIGHT, border=4)
        filterBox.Add(self.dataMax, proportion=1, flag=wx.LEFT | wx.RIGHT, border=4)
        filterBox.Add(self.numOre, proportion=1, flag=wx.LEFT | wx.RIGHT, border=5)
        vbox.Add(filterBox, proportion=0, flag=wx.EXPAND | wx.ALL, border=10)

        eventHBox = wx.BoxSizer(wx.HORIZONTAL)
        eventLabel = wx.StaticText(leftpanel, label="Eventi")
        eventHBox.Add(eventLabel, proportion=0, flag=wx.ALIGN_LEFT | wx.RIGHT, border=5)
        self.eventList = wx.ListCtrl(leftpanel, style=wx.LC_REPORT | wx.LC_HRULES)
        self.eventList.EnableCheckBoxes()
        self.eventList.InsertColumn(0, "Nome", width=150)
        self.eventList.InsertColumn(1, "Data", width=80)
        self.eventList.InsertColumn(2, "Ora", width=50)
        self.eventList.InsertColumn(3, "Numero ore", width=80)
        eventHBox.Add(self.eventList, proportion=1, flag=wx.EXPAND | wx.LEFT | wx.RIGHT, border=10)

        vbox.Add(eventHBox, proportion=0, flag=wx.EXPAND | wx.ALL, border=10)
        leftpanel.SetSizer(vbox, wx.EXPAND)
        svbox.Add(leftpanel, 1, wx.EXPAND)

        rightPanel = wx.Panel(self)

        buttonbox = wx.BoxSizer(wx.VERTICAL)
        self.calButton = wx.Button(rightPanel, label='Carica')
        self.calButton.Bind(wx.EVT_BUTTON, self.__load_button_press)
        buttonbox.Add(self.calButton, proportion=0, flag=wx.TOP | wx.BOTTOM | wx.RIGHT, border=10)
        self.filterButton = wx.Button(rightPanel, label="Filtra")
        buttonbox.Add(self.filterButton, proportion=0, flag=wx.TOP | wx.BOTTOM, border=10)
        self.filterButton.Bind(wx.EVT_BUTTON, self.__filter_button_press)
        self.filterButton.Disable()
        config.calModifyButton = wx.Button(rightPanel, label='Modifica')
        config.calModifyButton.Bind(wx.EVT_BUTTON, self.__cal_modify_button_press)
        config.calModifyButton.Disable()
        buttonbox.Add(config.calModifyButton, proportion=0, flag=wx.TOP | wx.BOTTOM, border=10)
        self.saveButton = wx.Button(rightPanel, label="Salva")
        self.saveButton.Bind(wx.EVT_BUTTON, self.__save_button_press)
        self.saveButton.Disable()
        buttonbox.Add(self.saveButton, proportion=0, flag=wx.TOP | wx.BOTTOM, border=10)
        self.selectAll = wx.Button(rightPanel, label="Seleziona Tutti")
        self.selectAll.Bind(wx.EVT_BUTTON, self.__checkAll)
        self.selectAll.Disable()
        buttonbox.Add(self.selectAll, proportion=0, flag=wx.TOP | wx.BOTTOM, border=10)
        self.deselectall = wx.Button(rightPanel, label="Deseleziona Tutti")
        self.deselectall.Bind(wx.EVT_BUTTON, self.__uncheckAll)
        self.deselectall.Disable()
        buttonbox.Add(self.deselectall, proportion=0, flag=wx.TOP | wx.BOTTOM, border=10)

        rightPanel.SetSizer(buttonbox, wx.EXPAND)
        svbox.Add(rightPanel, 0, wx.EXPAND)

        self.SetSizer(svbox)

    def __handle_keypress(self, event):
        keycode = event.GetKeyCode()
        if keycode < 255:
            # valid ASCII
            if chr(keycode).isnumeric() or keycode == 8:
                # Valid numeric character
                event.Skip()

    def __load_button_press(self, event):
        if self.link.IsEmpty() is not True and validators.url(self.link.GetValue()):
            url = self.link.GetValue()
            req = Request(url)
            response = urlopen(req)
            data = response.read()
            self.cal = Calendar.from_ical(data)
            index = 0
            key_events = {}
            if not self.eventList.IsEmpty():
                self.eventList.DeleteAllItems()
            for event in self.cal.walk('vevent'):
                if event.get('rrule') is not None:
                    evName = str(event.get('summary'))
                    start_date = event.get('dtstart').dt.strftime("%d/%m/%Y")
                    start_hour = event.get('dtstart').dt.strftime("%H:%M")
                    delta = relativedelta(event.get('dtend').dt, event.get('dtstart').dt)
                    key_events[event.get('dtstart').dt] = (evName, start_date, start_hour, str(delta.hours))
                    reoccur = event.get('rrule').to_ical().decode('utf-8')
                    startR = None
                    nTimes = int(re.search("\d+", reoccur).group())
                    for i in range(0, nTimes - 1):
                        rule = rrule.rrulestr(reoccur, dtstart=event.get('dtstart').dt)
                        if i == 0:
                            startR = rule.after(event.get('dtstart').dt)
                        else:
                            startR = rule.after(startR)

                        start_date = startR.strftime("%d/%m/%Y")
                        start_hour = startR.strftime("%H:%M")
                        key_events[startR] = (evName, start_date, start_hour, str(delta.hours))
                else:
                    evName = str(event.get('summary'))
                    start_date = event.get('dtstart').dt.strftime("%d/%m/%Y")
                    start_hour = event.get('dtstart').dt.strftime("%H:%M")
                    delta = relativedelta(event.get('dtend').dt, event.get('dtstart').dt)
                    key_events[event.get('dtstart').dt] = (evName, start_date, start_hour, str(delta.hours))
            for i in sorted(key_events):
                for j in range(len(key_events[i])):
                    if j == 0:
                        self.eventList.InsertItem(index, key_events[i][j])
                    if j == 1:
                        self.eventList.SetItem(index, 1, key_events[i][j])
                    if j == 2:
                        self.eventList.SetItem(index, 2, key_events[i][j])
                    if j == 3:
                        self.eventList.SetItem(index, 3, key_events[i][j])
                index += 1
            config.log.AppendText("Calendar caricato.\n")
            self.filterButton.Enable()
            self.selectAll.Enable()
            self.deselectall.Enable()
        else:
            missLink = wx.MessageDialog(None, "Link calendar assente/errato", caption="Errore",
                                        style=wx.OK, pos=wx.DefaultPosition)
            if missLink.ShowModal() == wx.OK:
                return

    def __checkAll(self, event):
        for i in range(self.eventList.GetItemCount()):
            if not self.eventList.IsItemChecked(i):
                self.eventList.CheckItem(i)

    def __uncheckAll(self, event):
        for i in range(self.eventList.GetItemCount()):
            if self.eventList.IsItemChecked(i):
                self.eventList.CheckItem(i, False)

    def __filter_button_press(self, event):
        if not self.eventList.IsEmpty():
            itemsNum = self.eventList.GetItemCount() - 1
            name = dateMin = dateMax = hours = None
            if not self.nameFilter.IsEmpty():
                name = self.nameFilter.GetValue()
            if not self.numOre.IsEmpty() and self.numOre.GetValue().isnumeric():
                hours = self.numOre.GetValue()
            if not self.dataMin.IsEmpty() and config.checkDate(self.dataMin.GetValue()):
                dateMin = datetime.strptime(self.dataMin.GetValue(), "%d/%m/%Y")
            if not self.dataMax.IsEmpty() and config.checkDate(self.dataMax.GetValue()):
                dateMax = datetime.strptime(self.dataMax.GetValue(), "%d/%m/%Y")
            while itemsNum >= 0:
                eliminate = False
                if name is not None:
                    if re.search(name, self.eventList.GetItemText(itemsNum, 0), re.IGNORECASE) is None:
                        eliminate = True
                if dateMin is not None:
                    listDate = datetime.strptime(self.eventList.GetItemText(itemsNum, 1), "%d/%m/%Y")
                    if listDate < dateMin:
                        eliminate = True
                if dateMax is not None:
                    listDate = datetime.strptime(self.eventList.GetItemText(itemsNum, 1), "%d/%m/%Y")
                    if listDate > dateMax:
                        eliminate = True
                if hours is not None:
                    if self.eventList.GetItemText(itemsNum, 3) != hours:
                        eliminate = True
                if eliminate:
                    self.eventList.DeleteItem(itemsNum)
                itemsNum -= 1
        else:
            missCal = wx.MessageDialog(None, "Caricare calendar", caption="Errore",
                                       style=wx.OK, pos=wx.DefaultPosition)
            if missCal.ShowModal() == wx.OK:
                return

    def __cal_modify_button_press(self, event):
        activity = config.acSelect.GetStringSelection()
        multiplier = config.multiplier.GetStringSelection()
        if not self.eventList.IsEmpty():
            checked = False
            for i in range(self.eventList.GetItemCount()):
                if self.eventList.IsItemChecked(i):
                    checked = True
                    break
            if checked:
                for i in range(self.eventList.GetItemCount()):
                    if self.eventList.IsItemChecked(i):
                        date = self.eventList.GetItemText(i, 1)
                        hours = self.eventList.GetItemText(i, 3)
                        config.file.modify(activity, date, int(hours), float(multiplier))
                modified = wx.MessageDialog(None, "File modificato", caption="Info",
                                            style=wx.OK, pos=wx.DefaultPosition)
                if modified.ShowModal() == wx.OK:
                    return
                if config.file.checkMissing():
                    message = config.file.getMissing()
                    config.log.AppendText(message + "\n")
                self.saveButton.Enable()
            else:
                missCal = wx.MessageDialog(None, "Selezionare 1 o più eventi", caption="Errore",
                                           style=wx.OK, pos=wx.DefaultPosition)
                if missCal.ShowModal() == wx.OK:
                    return
        else:
            missCal = wx.MessageDialog(None, "Caricare calendar", caption="Errore",
                                       style=wx.OK, pos=wx.DefaultPosition)
            if missCal.ShowModal() == wx.OK:
                return

    def __save_button_press(self, event):
        file = config.file
        saveFileDialog = wx.FileDialog(self, "Save", "", config.file.getFileName(), "Exel files (*.xlsx)|*.xlsx",
                                       wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if saveFileDialog.ShowModal() == wx.ID_CANCEL:
            return
        file.save(saveFileDialog.GetPath())



import wx
from . import config
from TimesheetExcelEditor.RangePanel import RangePanel
from TimesheetExcelEditor.ListPanel import ListPanel
from TimesheetExcelEditor.FormPanel import FormPanel
from TimesheetExcelEditor.CalendarPanel import CalendarPanel
from TimesheetExcelEditor.CreatePanel import CreatePanel
from TimesheetExcelEditor.Excel import Excel


class MyPanel(wx.Panel):

    def __init__(self, parent):
        super().__init__(parent)

        self.vbox = wx.BoxSizer(wx.VERTICAL)

        filebox = wx.BoxSizer(wx.HORIZONTAL)
        excel = wx.StaticText(self, label="File Excel")
        filebox.Add(excel, proportion=0, flag=wx.LEFT | wx.RIGHT | wx.TOP, border=5)
        self.fPath = wx.TextCtrl(self, style=wx.TE_READONLY)
        filebox.Add(self.fPath, proportion=1, flag=wx.ALIGN_TOP | wx.LEFT | wx.RIGHT | wx.TOP, border=5)
        self.loadButton = wx.Button(self, label='Scegli file')
        self.loadButton.Bind(wx.EVT_BUTTON, self.__load_button_press)
        filebox.Add(self.loadButton, proportion=0, flag=wx.LEFT | wx.RIGHT | wx.TOP, border=5)
        self.vbox.Add(filebox, proportion=0, flag=wx.EXPAND | wx.TOP, border=5)

        activityBox = wx.BoxSizer(wx.HORIZONTAL)
        acl = wx.StaticText(self, label="Attività")
        activityBox.Add(acl, proportion=0, flag=wx.LEFT, border=15)
        config.acSelect = wx.ComboBox(self, style=wx.CB_DROPDOWN | wx.CB_READONLY)
        activityBox.Add(config.acSelect, proportion=1, flag=wx.EXPAND | wx.LEFT, border=10)
        config.acSelect.Disable()
        ml = wx.StaticText(self, label="Moltiplicatore")
        activityBox.Add(ml, proportion=0, flag=wx.LEFT, border=10)
        config.multiplier = wx.ComboBox(self, choices=["1", "1.25", "1.5", "1.75", "2"],
                                        style=wx.CB_DROPDOWN | wx.CB_READONLY)
        activityBox.Add(config.multiplier, proportion=1, flag=wx.EXPAND | wx.LEFT | wx.RIGHT, border=10)
        config.multiplier.SetSelection(0)
        self.vbox.Add(activityBox, proportion=0, flag=wx.EXPAND | wx.TOP, border=10)

        tabBox = wx.BoxSizer(wx.HORIZONTAL)
        self.notebook = wx.Notebook(self)
        form = FormPanel(self.notebook)
        listTab = ListPanel(self.notebook)
        calTab = CalendarPanel(self.notebook)
        rangeTab = RangePanel(self.notebook)
        self.notebook.AddPage(form, "GIORNO")
        self.notebook.AddPage(listTab, "LISTA GIORNI")
        self.notebook.AddPage(rangeTab, "INTERVALLO")
        self.notebook.AddPage(calTab, "CALENDAR")
        self.notebook.AddPage(CreatePanel(self.notebook), "CREA")
        tabBox.Add(self.notebook, proportion=1, flag=wx.EXPAND | wx.TOP, border=5)
        self.vbox.Add(tabBox, proportion=1, flag=wx.EXPAND | wx.TOP, border=5)

        logBox = wx.BoxSizer(wx.HORIZONTAL)
        config.log = wx.TextCtrl(self, style=wx.TE_READONLY | wx.TE_MULTILINE | wx.SUNKEN_BORDER)
        logBox.Add(config.log, proportion=1, flag=wx.EXPAND | wx.TOP, border=5)
        self.vbox.Add(logBox, proportion=1, flag=wx.EXPAND)
        self.SetSizer(self.vbox)

    def __load_button_press(self, event):
        openFileDialog = wx.FileDialog(self, "Open", "", "", "Exel files (*.xlsx)|*.xlsx",
                                       wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        if openFileDialog.ShowModal() == wx.ID_CANCEL:
            return
        directory, filename = openFileDialog.GetDirectory(), openFileDialog.GetFilename()
        config.file = Excel()
        config.file.load(directory, filename)
        self.fPath.SetValue(config.file.getPath())
        if not config.log.IsEmpty():
            config.log.Clear()
        config.log.AppendText("File "+config.file.getFileName()+" caricato.\n")
        config.formModifyButton.Enable()
        config.listModifyButton.Enable()
        config.calModifyButton.Enable()
        config.rangeModifyButton.Enable()
        lis = config.file.getHeaders()
        config.acSelect.Enable()
        for keys in lis:
            config.acSelect.Append(lis[keys])
        config.acSelect.SetSelection(0)



import wx
from TimesheetExcelEditor.MyPanel import MyPanel


class MyFrame(wx.Frame):

    def __init__(self):
        super().__init__(None, title="Timesheet Excel Editor", size=(800, 600),
                         style=wx.DEFAULT_FRAME_STYLE ^ wx.RESIZE_BORDER)
        self.CenterOnScreen()
        self.EnableMaximizeButton(False)
        panel = MyPanel(self)
        self.Show()


def main():
    app = wx.App()
    frame = MyFrame()
    app.MainLoop()



import openpyxl
import re
import os
from . import config
from datetime import date as dt, timedelta
from openpyxl.utils import get_column_letter
from openpyxl import formatting
from openpyxl.styles import Font, Border, Side, Alignment, PatternFill


class Excel:

    def __init__(self):
        self.filename = None
        self.filePath = None
        self.wb = None
        self.headers = {}
        self.missing = []

    def load(self, directory, name):
        self.filename = name
        self.filePath = os.path.join(directory, name)
        self.wb = openpyxl.load_workbook(self.filePath)

    def createWorkbook(self, year, add):
        self.wb = openpyxl.Workbook()
        self.wb.remove(self.wb['Sheet'])
        self.__createSheet(year)
        for times in range(1, add+1):
            newYear = str(int(year)+times)
            self.__createSheet(newYear)
        return self.wb

    def __createSheet(self, year):
        self.wb.create_sheet(year)
        ws = self.wb[year]
        ws['A2'] = "RIEPILOGO " + str(int(year) - 1)
        ws['A4'] = "TOTALE ORE"
        ws['A5'] = "Attività istituzionale"
        ws['U2'] = "Previsione"
        number = 1
        cursiveFont = Font(italic=True)
        for acRow in range(6, 6+8):
            column = 'A'
            ws[column+str(acRow)] = "Progetto " + str(number)
            ws[column+str(acRow)].font = cursiveFont
            number += 1
        ws['A15'] = "TOTALE"
        months = ['G', 'F', 'M', 'A', 'M', 'G', 'L', 'A', 'S', 'O', 'N', 'D', 'TOT']
        monthsIndex = 0
        row = 21
        increment = 0
        boldFont = Font(bold=True)
        border = Border(left=Side(border_style="thin"), right=Side(border_style="thin"), top=Side(border_style="thin"),
                        bottom=Side(border_style="thin"))
        for r in range(3, 16):
            ws['A'+str(r)].border = border
        for col in range(2, 15):
            for rows in range(3, 14):
                if col != 14:
                    if rows == 3:
                        ws.cell(row=rows, column=col).value = months[monthsIndex]
                        ws.cell(row=rows, column=col).border = border
                        ws.cell(row=rows, column=col + 19).value = months[monthsIndex]
                        ws.cell(row=rows, column=col + 19).border = border
                        monthsIndex += 1
                    elif rows == 4:
                        ws.cell(row=rows, column=col).value = 125
                        ws.cell(row=rows, column=col).border = border
                        cell = ws.cell(row=rows, column=col).coordinate
                        ws.cell(row=rows, column=col + 19).value = "=" + cell
                        ws.cell(row=rows, column=col + 19).border = border
                    else:
                        column = 'AG'
                        if increment == 0:
                            ws.cell(row=rows, column=col).value = "="+column+str(row)
                            ws.cell(row=rows, column=col).border = border
                            c = ws.cell(row=rows, column=col).coordinate
                            ws.cell(row=rows, column=col + 19).value = "=" + c
                            ws.cell(row=rows, column=col + 19).border = border
                            increment += 1
                        else:
                            tempRow = row
                            ws.cell(row=rows, column=col).value = "=" + column + str(tempRow+increment)
                            ws.cell(row=rows, column=col).border = border
                            increment = (increment + 1) % 9
                            c = ws.cell(row=rows, column=col).coordinate
                            ws.cell(row=rows, column=col + 19).value = "=" + c
                            ws.cell(row=rows, column=col + 19).border = border
                else:
                    if rows == 3:
                        ws.cell(row=rows, column=col).value = months[monthsIndex]
                        ws.cell(row=rows, column=col).font = boldFont
                        ws.cell(row=rows, column=col).border = border
                        ws.cell(row=rows, column=col + 19).value = months[monthsIndex]
                        ws.cell(row=rows, column=col + 19).font = boldFont
                        ws.cell(row=rows, column=col + 19).border = border
                    else:
                        start = ws.cell(row=rows, column=2).coordinate
                        end = ws.cell(row=rows, column=col-1).coordinate
                        pStart = ws.cell(row=rows, column=21).coordinate
                        pEnd = ws.cell(row=rows, column=col - 1+19).coordinate
                        ws.cell(row=rows, column=col).value = "=SUM("+start+":"+end+")"
                        ws.cell(row=rows, column=col).font = boldFont
                        ws.cell(row=rows, column=col).border = border
                        ws.cell(row=rows, column=col+19).value = "=SUM(" + pStart + ":" + pEnd + ")"
                        ws.cell(row=rows, column=col + 19).font = boldFont
                        ws.cell(row=rows, column=col + 19).border = border
            row += 14
            ws['A15'].font = boldFont
            ws['A15'].border = border
            tot = ws.cell(row=4, column=col).coordinate
            start = ws.cell(row=5, column=col).coordinate
            end = ws.cell(row=14, column=col).coordinate
            ws.cell(row=15, column=col).value = "="+tot+"-SUM(" + start + ":" + end + ")"
            ws.cell(row=15, column=col).font = boldFont
            ws.cell(row=15, column=col).border = border
            pTot = ws.cell(row=4, column=col+19).coordinate
            pStart = ws.cell(row=5, column=col+19).coordinate
            pEnd = ws.cell(row=14, column=col+19).coordinate
            ws.cell(row=15, column=col + 19).value = "="+pTot+"-SUM(" + pStart + ":" + pEnd + ")"
            ws.cell(row=15, column=col + 19).font = boldFont
            ws.cell(row=15, column=col + 19).border = border
        ws['A17'] = "DETTAGLIO MENSILE"
        monthsDic = {"01": "GENNAIO", "02": "FEBBRAIO", "03": "MARZO", "04": "APRILE", "05": "MAGGIO", "06": "GIUGNO",
                     "07": "LUGLIO", "08": "AGOSTO", "09": "SETTEMBRE", "10": "OTTOBRE", "11": "NOVEMBRE",
                     "12": "DICEMBRE"}
        rowStart = 20
        for keys in monthsDic:
            column = 'A'
            ws.merge_cells('B'+str(rowStart-2)+':'+'D'+str(rowStart-2))
            ws['B'+str(rowStart-2)] = monthsDic[keys]
            ws['A'+str(rowStart-1)].border = border
            monthStart, monthEnd = self.__getMonthDays(int(year), keys, ws)
            for c in range(monthStart[1], monthEnd+1):
                ws.cell(row=rowStart-1, column=c).value = c - 1
                ws.cell(row=rowStart-1, column=c).border = border
                ws.cell(row=rowStart - 1, column=c).alignment = Alignment(horizontal='center')
                day = dt(int(year), int(keys), c - 1).strftime("%A")
                if day == "Saturday" or day == "Sunday":
                    color = None
                    if int(year) % 2 == 0:
                        color = PatternFill(start_color="FFA6A6A6", end_color="FFA6A6A6", fill_type="solid")
                    else:
                        color = PatternFill(start_color="FF8EB4E3", end_color="FF8EB4E3", fill_type="solid")
                    for row in range(rowStart - 1, rowStart + 11):
                        ws.cell(row=row, column=c).fill = color
            ws['AG'+str(rowStart-1)] = "TOT"
            ws['AG'+str(rowStart-1)].alignment = Alignment(horizontal='center')
            number = 6
            terminate = rowStart + 11
            first = True
            while rowStart < terminate:
                if first:
                    cellRange = 'B'+str(rowStart+10)+':AG'+str(rowStart+10)
                    lightRed = PatternFill(start_color="FFCCCB", end_color="FFCCCB", fill_type="solid")
                    ws.conditional_formatting.add(cellRange, formatting.rule.CellIsRule(operator='lessThan',
                                                                                        formula=['0'], fill=lightRed))
                    for c in range(2, 33):
                        totHours = ws.cell(row=rowStart, column=c).coordinate
                        startS = ws.cell(row=rowStart+1, column=c).coordinate
                        endS = ws.cell(row=rowStart+9, column=c).coordinate
                        ws.cell(row=rowStart+10, column=c).value = "=" + totHours + "-SUM(" + startS + ":" + endS + ")"
                    first = False
                if terminate-1 - rowStart == 10:
                    ws[column + str(rowStart)] = "TOTALE ORE"
                elif terminate-1 - rowStart == 9:
                    ws[column + str(rowStart)] = "Attività istituzionale"
                elif terminate-1 - rowStart != 0:
                    ws[column + str(rowStart)] = "="+column + str(number)
                    number += 1
                else:
                    ws[column + str(rowStart)] = "RESIDUO"
                ws['AG' + str(rowStart)] = '=SUM(' + 'B' + str(rowStart) + ':' + 'AF' + str(rowStart) + ')'
                rowStart += 1
            rowStart += 3
        ws.column_dimensions['A'].width = 20
        for c in range(2, 34):
            ws.column_dimensions[get_column_letter(c)].width = 4
        ws.column_dimensions['N'].width = 5
        ws.column_dimensions['AG'].width = 5
        return ws

    def getFileName(self):
        return self.filename

    def getPath(self):
        return self.filePath

    def checkMissing(self):
        return len(self.missing)

    def getMissing(self):
        message = "Anno/i "
        for y in self.missing:
            message += y + ", "
        message += "non trovato/i"
        self.missing.clear()
        return message

    def save(self, path):
        self.wb.save(path)
        config.log.AppendText("File salvato in "+path+"\n")

    def getHeaders(self):
        ws = self.wb.active
        self.headers["A1"] = "Attività istituzionale"
        for row in ws.iter_rows(max_col=1):
            for cell in row:
                m = re.match("Progetto \d+", str(cell.value))
                if m is not None:
                    self.headers[cell.coordinate] = m.group()
        return self.headers

    def __checkSheet(self, year):
        sheets = self.wb.sheetnames
        present = False
        for s in sheets:
            if s == year:
                present = True
        if not present:
            if year not in self.missing:
                self.missing.append(year)
        return present

    def __getMonthDays(self, year, m, ws):
        monthsDic = {"01": "GENNAIO", "02": "FEBBRAIO", "03": "MARZO", "04": "APRILE", "05": "MAGGIO", "06": "GIUGNO",
                     "07": "LUGLIO", "08": "AGOSTO", "09": "SETTEMBRE", "10": "OTTOBRE", "11": "NOVEMBRE",
                     "12": "DICEMBRE"}
        for months in ws.iter_rows(min_col=2, max_col=2):
            for cell in months:
                if cell.value is not None:
                    value = str(cell.value)
                    for keys in monthsDic:
                        if m == keys and re.search(monthsDic[keys], value, re.IGNORECASE):
                            start = (cell.row + 1, cell.column)
                            n = int(keys)
                            if n == 2:
                                if year % 4 == 0 and year % 100 != 0 or year % 400 == 0:
                                    end = cell.column + 28
                                else:
                                    end = cell.column + 27
                            elif n == 1 or n == 3 or n == 5 or n == 7 or n == 8 or n == 10 or n == 12:
                                end = cell.column + 30
                            else:
                                end = cell.column + 29
                            return start, end

    def modify(self, activity, data, nore, multiplier):
        numOre = nore
        day, month, year = data.split("/")
        date = dt(int(year), int(month), int(day))
        if self.__checkSheet(year):
            ws = self.wb[year]
            start, end = self.__getMonthDays(int(year), month, ws)
            for days in ws.iter_rows(min_row=start[0], max_row=start[0], min_col=start[1], max_col=end):
                for cell in days:
                    if cell.value == int(day):
                        for activities in ws.iter_rows(min_row=cell.row + 1, max_col=1):
                            for cell2 in activities:
                                value = None
                                for keys in self.headers:
                                    if self.headers[keys] == activity:
                                        value = "=" + keys
                                if cell2.value == value or cell2.value == activity:
                                    point = cell.column_letter + str(cell2.row)
                                    if ws[point].value is not None:
                                        ws[point] = ws[point].value + round(numOre * multiplier)
                                    else:
                                        ws[point] = round(numOre * multiplier)
                                    config.log.AppendText("Aggiunte "+str(round(numOre * multiplier)) +
                                                          " ore di "+activity+" alla data "+data + ".\n")
                                    if ws[point].value > 7:
                                        if not date.month == date.day == 1 and date.year == int(year):
                                            plus = ws[point].value - 7
                                            config.log.AppendText("Limite 7 ore superato, "+str(plus) +
                                                                  " ore in eccesso " +
                                                                  "ridistribuite nei giorni precedenti.\n")
                                            ws[point] = 7
                                            new_row = cell2.row
                                            new_col = cell.column - 1
                                            new_date = date - timedelta(days=1)
                                            while plus > 0 and (new_date.year == date.year):
                                                if new_date.month == date.month - 1:
                                                    null, new_col = self.__getMonthDays(int(year),
                                                                                        new_date.strftime("%m"), ws)
                                                    new_row = null[0] + (cell2.row - start[0])
                                                    date = new_date
                                                if new_date.weekday() != 6:
                                                    if ws.cell(new_row, new_col).value is not None:
                                                        ws.cell(new_row, new_col).value += 1
                                                    else:
                                                        ws.cell(new_row, new_col).value = 1
                                                    plus -= 1
                                                    if ws.cell(new_row, new_col).value == 7:
                                                        if not new_date.month == new_date.day == 1:
                                                            new_col -= 1
                                                            new_date = new_date - timedelta(days=1)
                                                else:
                                                    new_col -= 1
                                                    new_date = new_date - timedelta(days=1)
                                            if plus > 0:
                                                if ws.cell(new_row, new_col).value is not None:
                                                    ws.cell(new_row, new_col).value += plus
                                                else:
                                                    ws.cell(new_row, new_col).value = plus
                                    return True
